function Distillation_sfcn2(block)
% This is a level 2 sfcn in simulink describing the production of biodiesel
% through reactive distillation 
% The compounds are always listed in the following order: metOH/TG/DG/MG/GL/BD


%% Establishing the number of inputs and outputs:
nu = 6;
ny = 7;
Ntrays = 20;
NC = 6;

% block definitions
block.NumInputPorts  = nu;
block.NumOutputPorts = ny;

% Setup port properties to be inherited or dynamic
block.SetPreCompInpPortInfoToDynamic;
block.SetPreCompOutPortInfoToDynamic;

for i = 1:nu
    block.InputPort(i).Dimensions = 1;
    block.InputPort(i).DirectFeedthrough = true;
end
block.InputPort(2).Dimensions = NC; %The second input port is the feed molar fraction


for i = 1:ny
    block.OutputPort(i).Dimensions = 1;
end

block.OutputPort(1).Dimensions = NC;
block.OutputPort(2).Dimensions = NC;
block.OutputPort(3).Dimensions = (Ntrays+2)*(NC-1);
block.OutputPort(4).Dimensions = Ntrays+2;
block.OutputPort(5).Dimensions = Ntrays+2;
block.OutputPort(6).Dimensions = Ntrays+1;
block.OutputPort(7).Dimensions = Ntrays+1;

block.SampleTimes = [0 0];

% Setup Dwork
block.NumContStates = (Ntrays+2)*(NC-1)+Ntrays+2;

block.RegBlockMethod('InitializeConditions',    @InitializeConditions);
block.RegBlockMethod('Outputs',                 @Outputs);
block.RegBlockMethod('Derivatives',             @Derivatives);
block.RegBlockMethod('SetInputPortSamplingMode',@SetInpPortFrameData);


function SetInpPortFrameData(block, idx, fd)
  
block.InputPort(idx).SamplingMode = fd;
for i = 1:block.NumOutputPorts
    block.OutputPort(i).SamplingMode = fd;
end


function InitializeConditions(block)

load steadystate % x0
block.ContStates.Data = steadystate;

function Outputs(block)

% Important Parameters
Ntrays = 20;                        %Number of trays in column
NC = 6;                             %Number of compounds
Mw = [32.0 879.3844 616.95 354.52 92.094 294.47]; %Molecular weights [kg/kmol]
Ntrays_react = 3;                   %Number of reactive trays
alpha = [1 0.01 0.01 0.02 0.05 0.03];             %Relative volatilities 
Dc = 1.8564;                        %Diameter of column [m]
lw = 1.4295;                        %Length of weir [m]
hw = 0.20;                          %Heigth of weir [m]
Tb = [337.85 895.3 942.6 758.2 561.0 619.15];     %Boiling points [K]
q = 1.028;                          %Fraction of liquid in feed

%% Key Variables
% State variables
x = block.ContStates.Data(1:(Ntrays+2)*(NC-1)); %Molar fractions
H = block.ContStates.Data(((Ntrays+2)*(NC-1)+1):NC*(Ntrays+2)); %Molar holdup
xL = reshape(x,Ntrays+2,5);  %Reshaping into a matrix
HL = reshape(H,Ntrays+2,1);  %Reshaping into a vector
%We still use the sequence: MetOH/TG/DG/MG/GL/BD

% Disturbances
F0   = block.InputPort(1).Data;
x_in  = block.InputPort(2).Data;

% Manipulation
D   = block.InputPort(3).Data;
Vb   = block.InputPort(4).Data;
Ref   = block.InputPort(5).Data;
B   = block.InputPort(6).Data;

%% Calculation of compositions and temperature by method 1

T = 373.15*ones(Ntrays+2,1); %Setting the initial temperature
XX = zeros(Ntrays+2,NC);  x_BD = zeros(Ntrays+2,1);
y = zeros(Ntrays+2,NC-1); y_BD = zeros(Ntrays+2,1);

for i = 1:Ntrays+2
    %Molar fraction of BD:
    x_BD(i,1) = 1 - sum(xL(i,:));
    XX(i,:) = [xL(i,:) x_BD(i,1)];
    
    %Temperature by Method 1:
    % T(i,1) = sum(Tb.*XX(i,:));  %The temperature on each tray is the sum of the individual boiling points
    
    %Vapour composition by relative volatilities:
%      yy = (alpha.*XX(i,:))/sum(alpha.*XX(i,:));
%      y(i,1) = yy(1,1); y(i,2) = yy(1,2); y(i,3) = yy(1,3); y(i,4) = yy(1,4); y(i,5) = yy(1,5);
% %     
% %     %Temperature by Method 2:
%      T(i,1) = sum(((XX(i,:) + yy)/2).*Tb);  
end

%% Calculations of densities in the liquid phase:

% Initialising sizes
rho_MetOH = ones(Ntrays+2,1); rho_TG = ones(Ntrays+2,1); rho_GL = ones(Ntrays+2,1);
rho_BD = ones(Ntrays+2,1); rho_DG = ones(Ntrays+2,1); rho_MG = ones(Ntrays+2,1);
rho = [rho_MetOH rho_TG rho_DG rho_MG rho_GL rho_BD];
rho_l = ones(Ntrays+2,1); rho_L = ones(Ntrays+2,1); x_wt = zeros(Ntrays+2,NC);

%The constants
Ar_met = 2.3267; Br_met = 0.27073; Cr_met = 512.5; Dr_met = 0.24713;
Ar_TG = 0.026085; Br_TG = 0.14259; Cr_TG = 934.6; Dr_TG = 0.28571;
Ar_GL = 0.92382; Br_GL = 0.24386; Cr_GL = 850; Dr_GL = 0.22114;
Ar_BD = 0.20469; Br_BD = 0.23737; Cr_BD = 767.4; Dr_BD = 0.28571;
 
for i = 1:Ntrays+2
      rho_MetOH(i,1) = Ar_met/(Br_met^(1+(1-(T(i,1)/Cr_met))^Dr_met));
      rho_TG(i,1) = Ar_TG/(Br_TG^(1+(1-(T(i,1)/Cr_TG))^Dr_TG));
      rho_GL(i,1) = Ar_GL/(Br_GL^(1+(1-(T(i,1)/Cr_GL))^Dr_GL));
      rho_BD(i,1) = Ar_BD/(Br_BD^(1+(1-(T(i,1)/Cr_BD))^Dr_BD));
      rho_DG(i,1) = ((2/3)*rho_TG(i,1)) + ((1/3)*rho_BD(i,1)); %We assume that the density of DG and MG can be 
      rho_MG(i,1) = ((1/3)*rho_TG(i,1)) + ((2/3)*rho_BD(i,1)); %approximated by the densities of BD and TG
      rho(i,:) = [rho_MetOH(i,1) rho_TG(i,1) rho_DG(i,1) rho_MG(i,1) rho_GL(i,1) rho_BD(i,1)];
      rho_l(i,1) = sum(rho(i,:).*XX(i,:));          %A common liquid density for the tray [kmol/m3]
      x_wt(i,:) = (XX(i,:)*HL(i,1).*Mw)/sum(XX(i,:)*HL(i,1).*Mw); %Weight fraction 
      rho_L(i,1) = sum(rho(i,:).*Mw.*x_wt(i,:));    %A common liquid density for the tray [kg/m3]
end

%% Calculating the flowrates in the column
% Initializing sizes
V = ones(Ntrays+1,1);  %V is a vector of the molar vapor flows in the column
L = ones(Ntrays+1,1);  %L is a vector of the different trays in the column.

% Applying Francis weir for the liquid flowrates
how = ones(Ntrays,1); Lw = ones(Ntrays,1);
for i = 1:Ntrays
      how(i,1) = (HL(i+1,1)-(pi*hw*rho_l(i+1,1)*0.88*(Dc/2)^2))/(pi*0.88*(Dc/2)^2);    %Calculation of how
      if how(i,1) <0
          Lw(i+1,1) = 0;
          L(i+1,1) = 0;
      else          
          Lw(i+1,1) = ((how(i,1)*10^3)/750)^1.5*rho_L(i+1,1)*lw;    %Francis weir [kg/s]
          L(i+1,1) = (Lw(i+1,1)*3600)/sum(XX(i+1,:).*Mw);           %Liquid flow [kmol/h]
      end
end

% Vapour flowrates
V(Ntrays+1,1) = Vb;

% Assuming constant molar overflow for vapour flowrates
for i = 2:Ntrays
    V(i,1) = V(Ntrays+1,1);
end

% Correction for the feed:
V(1,1) = V(2,1) + ((1-q)*F0); 

% Molar balances over the top and the bottom: 
L(1,1) = Ref;

%% Calculating the vapour compositions by vapour pressures, and temperature by method 2
P = ones(Ntrays+2,1);
%Parameters necessary to calculate vapour pressures:
    A_met = 82.718; B_met = -6.9045*10^3; C_met = -8.8622; D_met = 7.4664*10^-6; E_met = 2;
    A_TG = 234.71; B_TG = -3.4699*10^4; C_TG = -27.25; D_TG = 1.5475*10^-18; E_TG = 6;
    A_DG = -15.931; B_DG = -2111.0; C_DG = 2.4303; D_DG = 8.0567*10^-21; E_DG = 6;
    A_MG = 118.95; B_MG = -2.0181*10^4; C_MG = -14.32; D_MG = 9.148*10^-19; E_MG = 6;
    A_GL = 99.986; B_GL = -1.3808*10^4; C_GL = -10.088; D_GL = 3.5712*10^-19; E_GL = 6;
    A_BD = 105.47; B_BD = -1.4531*10^4; C_BD = -10.986; D_BD = 2.5735*10^-18; E_BD = 6;

for j = 1:Ntrays+2
    %Vapour pressure equations
    P_met = 9.869*(10^-6)*exp(A_met + (B_met/T(j,1)) + (C_met*log(T(j,1))) + (D_met*T(j,1)^E_met));
    P_TG = 9.869*(10^-6)*exp(A_TG + (B_TG/T(j,1)) + (C_TG*log(T(j,1))) + (D_TG*T(j,1)^E_TG));
    P_DG = 9.869*exp(A_DG + (B_DG/T(j,1)) + (C_DG*log(T(j,1))) + (D_DG*T(j,1)^E_DG));
    P_MG = 9.869*exp(A_MG + (B_MG/T(j,1)) + (C_MG*log(T(j,1))) + (D_MG*T(j,1)^E_MG));
    P_GL = 9.869*(10^-6)*exp(A_GL + (B_GL/T(j,1)) + (C_GL*log(T(j,1))) + (D_GL*T(j,1)^E_GL));
    P_BD = 9.869*(10^-6)*exp(A_BD + (B_BD/T(j,1)) + (C_BD*log(T(j,1))) + (D_BD*T(j,1)^E_BD));
    %Raoults law
    P(j,1) = P_met*xL(i,1) + P_TG*xL(i,2) + P_DG*xL(i,3) + P_MG*xL(i,4) + P_GL*xL(i,5) + P_BD*x_BD(i,1);
    y(j,1) = (P_met*xL(i,1))/P(j,1);
    y(j,2) = (P_TG*xL(i,2))/P(j,1);
    y(j,3) = (P_DG*xL(i,3))/P(j,1);
    y(j,4) = (P_MG*xL(i,4))/P(j,1);
    y(j,5) = (P_GL*xL(i,5))/P(j,1);
    y_BD(j,1) = 1- sum(y(j,:));
    yy = [y(j,:) y_BD(j,1)];
    
%     % Temperature by method 2:
    T(j,1) = sum(((XX(j,:) + yy)/2).*Tb);  
end


       

%% Determining distillate and bottoms flow
%Determination of the distillate flow
vD = D*XX(1,:);       % partial molar flows (kmol/h)

%Determination of the bottoms flow
vB = B*XX(Ntrays+2,:); % partial molar flows (kmol/h)


%% Output variables
block.OutputPort(1).Data = vD;
block.OutputPort(2).Data = vB;
block.OutputPort(3).Data = x;
block.OutputPort(4).Data = H;
block.OutputPort(5).Data = T;
block.OutputPort(6).Data = L;
block.OutputPort(7).Data = V;


function Derivatives(block)
%% Important parameters
Ntrays = 20;                %Number of trays
NC = 6;                     %Number of compounds
R = 8.314472*10^(-3);       %Gas constant [kJ/K*mol]
Mw = [32.0 879.3844 616.95 354.52 92.094 294.47]; %Molecular weights [kg/kmol]
Ntrays_react = 3;           %Number of reactive trays
Tb = [337.85 895.3 942.6 758.2 561.0 619.15];
q = 1.028;                  %Liquid feed fraction
Dc = 1.8564;                %Diameter of column [m]
lw = 1.4295;                %Length of weir [m]
hw = 0.20;                  %Heigth of weir [m]
alpha = [1 0.01 0.01 0.02 0.05 0.03];   %Relative volatilities 

%% Variables
% State variables
  x = block.ContStates.Data(1:(Ntrays+2)*(NC-1));
  H = block.ContStates.Data(((Ntrays+2)*(NC-1)+1):NC*(Ntrays+2));
  xL = reshape(x,Ntrays+2,NC-1);
  HL = reshape(H,Ntrays+2,1);

% Disturbances
  F0   = block.InputPort(1).Data;
  x_in = block.InputPort(2).Data;
%The input F0 is the flowrate [kmol/h] and x_in is the composition.
%The composition is given in the following order: MetOH/TG/DG/MG/GL/BD
  
% Manipulation 
  D   = block.InputPort(3).Data;
  Vb = block.InputPort(4).Data;
  Ref = block.InputPort(5).Data;
  B = block.InputPort(6).Data;
  
%% Calculation of temperatures and compositions

%Calculating the temperature on each tray:
T = 373.15*ones(Ntrays+2,1); %Setting the initial temperature
XX = zeros(Ntrays+2,NC);  x_BD = zeros(Ntrays+2,1);
y = zeros(Ntrays+2,NC-1); y_BD = zeros(Ntrays+2,1);

for i = 1:Ntrays+2
    x_BD(i,1) = 1 - sum(xL(i,:));
    XX(i,:) = [xL(i,:) x_BD(i,1)];
    
%     %Temperature by method 1:
%     T(i,1) = sum(Tb.*XX(i,:));
%     %Vapour phase composition by relative volatilities:
%     yy = (alpha.*XX(i,:))/sum(alpha.*XX(i,:));
%     y(i,1) = yy(1,1); y(i,2) = yy(1,2); y(i,3) = yy(1,3); y(i,4) = yy(1,4); y(i,5) = yy(1,5);
%     %Temperature by method 2: 
%     T(i,1) = sum(((XX(i,:) + yy)/2).*Tb);  
end

%% Calculations of densities in the liquid phase:

% Initialising sizes
rho_MetOH = ones(Ntrays+2,1); rho_TG = ones(Ntrays+2,1); rho_GL = ones(Ntrays+2,1);
rho_BD = ones(Ntrays+2,1); rho_DG = ones(Ntrays+2,1); rho_MG = ones(Ntrays+2,1);
rho = [rho_MetOH rho_TG rho_DG rho_MG rho_GL rho_BD];
rho_l = ones(Ntrays+2,1); rho_L = ones(Ntrays+2,1); x_wt = zeros(Ntrays+2,NC);

% The constants
Ar_met = 2.3267; Br_met = 0.27073; Cr_met = 512.5; Dr_met = 0.24713;
Ar_TG = 0.026085; Br_TG = 0.14259; Cr_TG = 934.6; Dr_TG = 0.28571;
Ar_GL = 0.92382; Br_GL = 0.24386; Cr_GL = 850; Dr_GL = 0.22114;
Ar_BD = 0.20469; Br_BD = 0.23737; Cr_BD = 767.4; Dr_BD = 0.28571;
 
for i = 1:Ntrays+2
    %Densities [kmol/m3]
      rho_MetOH(i,1) = Ar_met/(Br_met^(1+(1-(T(i,1)/Cr_met))^Dr_met)); 
      rho_TG(i,1) = Ar_TG/(Br_TG^(1+(1-(T(i,1)/Cr_TG))^Dr_TG));
      rho_GL(i,1) = Ar_GL/(Br_GL^(1+(1-(T(i,1)/Cr_GL))^Dr_GL));
      rho_BD(i,1) = Ar_BD/(Br_BD^(1+(1-(T(i,1)/Cr_BD))^Dr_BD));
      rho_DG(i,1) = ((2/3)*rho_TG(i,1)) + ((1/3)*rho_BD(i,1));  
      rho_MG(i,1) = ((1/3)*rho_TG(i,1)) + ((2/3)*rho_BD(i,1)); 
      rho(i,:) = [rho_MetOH(i,1) rho_TG(i,1) rho_DG(i,1) rho_MG(i,1) rho_GL(i,1) rho_BD(i,1)];
      %A common liquid density for the tray [kmol/m3]:
      rho_l(i,1) = sum(rho(i,:).*XX(i,:)); 
      %Weight fractions:
      x_wt(i,:) = (XX(i,:)*HL(i,1).*Mw)/sum(XX(i,:)*HL(i,1).*Mw); 
      %A common liquid density for the tray [kg/m3]
      rho_L(i,1) = sum(rho(i,:).*Mw.*x_wt(i,:)); 
end

%% Calculating the flowrates in the column
% Initializing sizes
V = ones(Ntrays+1,1);  %V is a vector of the molar vapor flows in the column
L = ones(Ntrays+1,1);  %L is a vector of the different trays in the column.

% Applying Francis weir for the liquid flowrates
how = ones(Ntrays,1); Lw = ones(Ntrays,1);
for i = 1:Ntrays
    %Calculation of height over weir [m]:
      how(i,1) = (HL(i+1,1)-(pi*hw*rho_l(i+1,1)*0.88*(Dc/2)^2))/(pi*0.88*(Dc/2)^2); 
      if how(i,1) <0
          %Ensures no negative flowrates within column
          Lw(i+1,1) = 0;
          L(i+1,1) = 0;
      else
          %Francis weir [kg/s]
          Lw(i+1,1) = ((how(i,1)*10^3)/750)^1.5*rho_L(i+1,1)*lw;    
          %Liquid flowrate [kmol/h]
          L(i+1,1) = (Lw(i+1,1)*3600)/sum(XX(i+1,:).*Mw);           
      end
end

% Vapour flowrates
V(Ntrays+1,1) = Vb;          

%Assume constant molar flowrate:
for i = 2:Ntrays
    V(i,1) = V(Ntrays+1,1);
end

% Correction for the feed:
V(1,1) = V(2,1) + ((1-q)*F0);

% Molar balances over the top: 
L(1,1) = Ref;   %The first liquid molar flow is equal to the reflux.
       

%% Calculating the vapour compositions by vapour pressures + temperature by method 2
 %Parameters necessary to calculate vapour pressures:
A_met = 82.718; B_met = -6.9045*10^3; C_met = -8.8622; D_met = 7.4664*10^-6; E_met = 2;
A_TG = 234.71; B_TG = -3.4699*10^4; C_TG = -27.25; D_TG = 1.5475*10^-18; E_TG = 6;
A_DG = -15.931; B_DG = -2111.0; C_DG = 2.4303; D_DG = 8.0567*10^-21; E_DG = 6;
A_MG = 118.95; B_MG = -2.0181*10^4; C_MG = -14.32; D_MG = 9.148*10^-19; E_MG = 6;
A_GL = 99.986; B_GL = -1.3808*10^4; C_GL = -10.088; D_GL = 3.5712*10^-19; E_GL = 6;
A_BD = 105.47; B_BD = -1.4531*10^4; C_BD = -10.986; D_BD = 2.5735*10^-18; E_BD = 6;
P = ones(Ntrays+2,NC);

for j = 1:Ntrays+2
    %Vapour pressure equations
    P_met = 9.869*(10^-6)*exp(A_met + (B_met/T(j,1)) + (C_met*log(T(j,1))) + (D_met*T(j,1)^E_met));
    P_TG = 9.869*(10^-6)*exp(A_TG + (B_TG/T(j,1)) + (C_TG*log(T(j,1))) + (D_TG*T(j,1)^E_TG));
    P_DG = 9.869*exp(A_DG + (B_DG/T(j,1)) + (C_DG*log(T(j,1))) + (D_DG*T(j,1)^E_DG));
    P_MG = 9.869*exp(A_MG + (B_MG/T(j,1)) + (C_MG*log(T(j,1))) + (D_MG*T(j,1)^E_MG));
    P_GL = 9.869*(10^-6)*exp(A_GL + (B_GL/T(j,1)) + (C_GL*log(T(j,1))) + (D_GL*T(j,1)^E_GL));
    P_BD = 9.869*(10^-6)*exp(A_BD + (B_BD/T(j,1)) + (C_BD*log(T(j,1))) + (D_BD*T(j,1)^E_BD));
    %Raoults law:
    P(j,1) = P_met*xL(i,1) + P_TG*xL(i,2) + P_DG*xL(i,3) + P_MG*xL(i,4) + P_GL*xL(i,5) + P_BD*x_BD(i,1);
    y(j,1) = (P_met*xL(i,1))/P(j,1);
    y(j,2) = (P_TG*xL(i,2))/P(j,1);
    y(j,3) = (P_DG*xL(i,3))/P(j,1);
    y(j,4) = (P_MG*xL(i,4))/P(j,1);
    y(j,5) = (P_GL*xL(i,5))/P(j,1);
    y_BD(j,1) = 1- sum(y(j,:));
    yy = [y(j,:) y_BD(j,1)];
    %Temperature by method 2:
    T(j,1) = sum(((XX(j,:) + yy)/2).*Tb);  
end
 
%% Calculation of the kinetics in the liquid phase:

%The sizes of the kinetic constants:
k1 = ones(Ntrays+2,1); k2 = ones(Ntrays+2,1); k3 = ones(Ntrays+2,1);
k4 = ones(Ntrays+2,1); k5 = ones(Ntrays+2,1); k6 = ones(Ntrays+2,1);

for i = 1:Ntrays+2
%Kinetic data for the reaction
k1(i) = 3.9*3600*(10^7)*exp(-54.9987/(R*T(i,1)));    %rate constants [m3/kmol*h]
k2(i) = 5.78*3600*(10^5)*exp(-41.5555/(R*T(i,1)));
k3(i) = 5.906*3600*(10^12)*exp(-83.0942/(R*T(i,1)));
k4(i) = 9.888*3600*(10^9)*exp(-61.2496/(R*T(i,1)));
k5(i) = 5.335*3600*(10^3)*exp(-26.8655/(R*T(i,1)));
k6(i) = 2.1*3600*(10^4)*exp(-40.1162/(R*T(i,1)));
end

%We need to convert the molar input to concentrations in order to calculate
%the reaction outputs:
C = zeros(Ntrays_react,6); 
R = zeros(Ntrays_react,6);
M = zeros(size(C));
Msum = zeros(Ntrays_react,1);
for i = 1:Ntrays_react  
   vH = HL(i+1,1)*XX(i+1,:);       %Molar holdup per component
   vr = sum(vH./rho(i+1,:));       %volume holdup
   C(i,:) = vH/vr;                 %concentration of each component
%The reactions. R is the gain/loss in [kmol/h]
R_TG = -(k1(i+1)*C(i,2)*C(i,1)) + (k2(i+1)*C(i,3)*C(i,6));
R_DG = (k1(i+1)*C(i,2)*C(i,1)) - (k2(i+1)*C(i,3)*C(i,6)) - (k3(i+1)*C(i,3)*C(i,1)) + (k4(i+1)*C(i,4)*C(i,6));
R_MG = (k3(i+1)*C(i,3)*C(i,1)) - (k4(i+1)*C(i,4)*C(i,6)) - (k5(i+1)*C(i,4)*C(i,1)) + (k6(i+1)*C(i,5)*C(i,6));
R_BD = (k1(i+1)*C(i,2)*C(i,1))-(k2(i+1)*C(i,3)*C(i,6))+(k3(i+1)*C(i,3)*C(i,1))-(k4(i+1)*C(i,4)*C(i,6))+(k5(i+1)*C(i,4)*C(i,1))-(k6(i+1)*C(i,5)*C(i,6));
R_GL = (k5(i+1)*C(i,4)*C(i,1)) - (k6(i+1)*C(i,5)*C(i,6));
R_MetOH = -(k1(i+1)*C(i,2)*C(i,1))+(k2(i+1)*C(i,3)*C(i,6))-(k3(i+1)*C(i,3)*C(i,1))+(k4(i+1)*C(i,4)*C(i,6))-(k5(i+1)*C(i,4)*C(i,1))+(k6(i+1)*C(i,5)*C(i,6));
%We store the molar gain of each component in an R matrix:
R(i,1) = R_MetOH; R(i,2) = R_TG; R(i,3) = R_DG; R(i,4) = R_MG; 
R(i,5) = R_GL; R(i,6) = R_BD;
M(i,:) = R(i,:)*vr; %The change in molar holdup of each component saved in a vector
Msum(i,1) = sum(M(i,:));
end
%The order of compounds is as before: MetOH/TG/DG/MG/GL/BD
                                    %    1  2  3  4  5  6


%% Molar balances in the column (the states)
dHxdt = zeros(Ntrays+2,5); 
dHdt = zeros(Ntrays+2,1);

% Column
i = 2:Ntrays_react+1;
dHdt(i,1) = L(i-1,1) - L(i,1) + V(i,1) - V(i-1,1) + Msum(i-1); 

i = Ntrays_react+2:Ntrays+1;
dHdt(i,1) = L(i-1) - L(i) + V(i,1) - V(i-1,1);

%Correction for the feed:
dHdt(2,1) = dHdt(2,1) + F0;


%The molar holdup composition balance for the feed tray:
for j = 1:5;
    for i = 2
        dHxdt(i,j) = ((F0*x_in(j,1)) + (V(i,1)*y(i,j)) + L(i-1,1)*xL(i-1,j) - (V(i-1,1)*y(i-1,j)) - (L(i,1)*xL(i,j)) + M(i-1,j));
    end
end

%The molar holdup composition balance for the two other reactive trays:
for j = 1:5;
    for i = 3:Ntrays_react+1
        dHxdt(i,j) = (V(i,1)*y(i,j)) + (L(i-1,1)*xL(i-1,j) - (V(i-1,1)*y(i-1,j)) - (L(i,1)*xL(i,j)) + M(i-1,j));   
    end
end

%The molar holdup composition balance for the non-reactive trays:
 for j = 1:5;
     for i = Ntrays_react+2:Ntrays+1;
        dHxdt(i,j) = (V(i,1)*y(i,j)) + (L(i-1,1)*xL(i-1,j)) - (V(i-1,1)*y(i-1,j)) - (L(i,1)*xL(i,j));
     end
 end

%Reboiler (assumed to be an equilibrium stage)
i = Ntrays+2;
dHdt(i,1) = L(Ntrays+1,1) - V(Ntrays+1,1) - B;

j=1:5;
dHxdt(i,j)= (L(Ntrays+1,1)*xL(Ntrays+1,j) - V(Ntrays+1,1)*y(Ntrays+2,j) - B*xL(Ntrays+2,j));

%Total condenser (not an equilibrium stage)
i = 1;
dHdt(i,1) = V(1,1) - L(1,1) - D;

j = 1:5;
dHxdt(i,j)= V(1,1)*y(i,j) - L(1,1)*xL(i,j) - D*xL(i,j);

%Computing the derivative for the mole fractions from d(Mx) = x dM + M dx
dxdt=(dHxdt - (xL.*(dHdt*ones(1,5))))./(HL*ones(1,5));

dxdtout = reshape(dxdt,(Ntrays+2)*5,1);
dxdtout = [dxdtout; dHdt];
     
     
  block.Derivatives.Data = dxdtout;

